import { __awaiter, __generator } from "tslib";
import * as tf from '@tensorflow/tfjs-core';
import { Point } from '../classes';
import { nonMaxSuppression } from '../ops';
import { extractImagePatches } from './extractImagePatches';
import { MtcnnBox } from './MtcnnBox';
import { ONet } from './ONet';
export function stage3(img, inputBoxes, scoreThreshold, params, stats) {
    return __awaiter(this, void 0, void 0, function () {
        var ts, onetInputs, onetOuts, scoresTensor, scores, _a, _b, indices, filteredRegions, filteredBoxes, filteredScores, finalBoxes, finalScores, points, indicesNms;
        var _this = this;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    ts = Date.now();
                    return [4 /*yield*/, extractImagePatches(img, inputBoxes, { width: 48, height: 48 })];
                case 1:
                    onetInputs = _c.sent();
                    stats.stage3_extractImagePatches = Date.now() - ts;
                    ts = Date.now();
                    onetOuts = onetInputs.map(function (onetInput) {
                        var out = ONet(onetInput, params);
                        onetInput.dispose();
                        return out;
                    });
                    stats.stage3_onet = Date.now() - ts;
                    scoresTensor = onetOuts.length > 1
                        ? tf.concat(onetOuts.map(function (out) { return out.scores; }))
                        : onetOuts[0].scores;
                    _b = (_a = Array).from;
                    return [4 /*yield*/, scoresTensor.data()];
                case 2:
                    scores = _b.apply(_a, [_c.sent()]);
                    scoresTensor.dispose();
                    indices = scores
                        .map(function (score, idx) { return ({ score: score, idx: idx }); })
                        .filter(function (c) { return c.score > scoreThreshold; })
                        .map(function (_a) {
                        var idx = _a.idx;
                        return idx;
                    });
                    return [4 /*yield*/, Promise.all(indices.map(function (idx) { return __awaiter(_this, void 0, void 0, function () {
                            var regionsData;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, onetOuts[idx].regions.array()];
                                    case 1:
                                        regionsData = _a.sent();
                                        return [2 /*return*/, new MtcnnBox(regionsData[0][0], regionsData[0][1], regionsData[0][2], regionsData[0][3])];
                                }
                            });
                        }); }))];
                case 3:
                    filteredRegions = _c.sent();
                    filteredBoxes = indices
                        .map(function (idx, i) { return inputBoxes[idx].calibrate(filteredRegions[i]); });
                    filteredScores = indices.map(function (idx) { return scores[idx]; });
                    finalBoxes = [];
                    finalScores = [];
                    points = [];
                    if (!(filteredBoxes.length > 0)) return [3 /*break*/, 5];
                    ts = Date.now();
                    indicesNms = nonMaxSuppression(filteredBoxes, filteredScores, 0.7, false);
                    stats.stage3_nms = Date.now() - ts;
                    finalBoxes = indicesNms.map(function (idx) { return filteredBoxes[idx]; });
                    finalScores = indicesNms.map(function (idx) { return filteredScores[idx]; });
                    return [4 /*yield*/, Promise.all(indicesNms.map(function (idx, i) {
                            return Promise.all(Array(5).fill(0).map(function (_, ptIdx) { return __awaiter(_this, void 0, void 0, function () {
                                var pointsData;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, onetOuts[idx].points.array()];
                                        case 1:
                                            pointsData = _a.sent();
                                            return [2 /*return*/, new Point(((pointsData[0][ptIdx] * (finalBoxes[i].width + 1)) + finalBoxes[i].left), ((pointsData[0][ptIdx + 5] * (finalBoxes[i].height + 1)) + finalBoxes[i].top))];
                                    }
                                });
                            }); }));
                        }))];
                case 4:
                    points = _c.sent();
                    _c.label = 5;
                case 5:
                    onetOuts.forEach(function (t) {
                        t.regions.dispose();
                        t.scores.dispose();
                        t.points.dispose();
                    });
                    return [2 /*return*/, {
                            boxes: finalBoxes,
                            scores: finalScores,
                            points: points
                        }];
            }
        });
    });
}
//# sourceMappingURL=stage3.js.map